/*--- CRC16.C --------------------- Listing 5 -------
*   Performs CRC-16 check on a file.                 
*   Modeled on Bill Hunt's 1987 article.              
*   Ported/Validated for Turbo C by Nigel Cort       
*   (c) William Hunt / C Gazette
*--------------------------------------------------*/

#include <stdio.h>

int crc16_table[16] =             /* CRC-16 for each nibble */
    { 
      0x0000 , 0xCC01 , 0xD801 , 0x1400 ,
      0xF001 , 0x3C00 , 0x2800 , 0xE401 ,
      0xA001 , 0x6C00 , 0x7800 , 0xB401 ,
      0x5000 , 0x9C01 , 0x8801 , 0x4400 
    } ;

int crc16 (                      /* Compute a CRC checksum */
    int table[] ,               /* CRC lookup table */
    int start ,                 /* starting value */
    char *p ,                   /* points to array of chars to total */
    int n )                     /* number of chars to total */
{
    register int total ;
    register int r1 ;

    total = start ;
    while ( n-- > 0 )                 /* process each byte */
    { 
        r1 = table[ total & 0xF ] ;     /* do low 4 bits */
        total = ( total >> 4 ) & 0x0FFF;
        total = total ^ r1 ^ table[ *p & 0xF ] ;

        r1 = table[ total & 0xF ] ;   /* do high 4 bits */
        total = ( total >> 4 ) & 0x0FFF;
        total = total ^ r1 ^ table[ ( *p >> 4 ) & 0xF ];

        p++ ;                         /* advance to next byte */
    }
    return ( total ) ;
}


#define RD_SIZE  8192
char buffer [ RD_SIZE ] ;

void main ( int argc, char *argv[] )
{
    FILE *fin ;             /* file descriptor */
    int n ;                 /* number of bytes actually read */
    int crc ;               /* CRC value */

    if ( argc < 2 )
    {
        fputs ( "USAGE - chkfile filename.ext\n", stderr ) ;
        exit ( 5 ) ;
    }

    if (( fin = fopen (argv[1], "rb")) == NULL )
    {
        fprintf ( stderr, "Cannot open %s \n", argv[1] ) ;
        exit ( 10 ) ;
    }

    crc = 0 ;

    do                                               /* Principal Loop      */
    {
        n = fread ( buffer, 1, RD_SIZE, fin );       /* Read file in blocks */
        crc = crc16 ( crc16_table, crc, buffer, n ); /* Add block to crc    */
    }
    while (n == RD_SIZE);                            /* Stop when done      */

    fclose (fin) ;

    if ( ferror ( fin ))                            /* Check for read error */
        printf ( "Error in Processing %s\n", argv[1] ) ;
    else                                            /* OK, report CRC       */
        printf ("CRC-16 for  %s = %04X ", argv[1], crc ) ;
}